"""some standard constants used with an uppercut system"""
class EnvVars(object):
    """commonly used env variable names"""
    SHARED_VAR_SERVER = "SHARED_VAR_SERVER" # the env var used to store the host name of the service that manages variables shared between modules.
    MOD_REG_SERVER = "MOD_REG_SERVER" # the env var used to indicate the host name of the module registration server

class Keys(object):
    """commonly used key names"""
    MOD_LIST = "UPPERCUT_MOD_LIST"
# some friendlier constant names.
SharedVariableServer = SHARED_VAR_SERVER
ModuleRegistrationServer = MOD_REG_SERVER


""" tools to configure modules used in an uppercut system """
from redis import Redis
import constants
import json

def Register(name,description):
    """ registers register the module's particulars on the in mem repo """
    _redis = Redis(host=constants.EnvVars.MOD_REG_SERVER)
    modInfo = {'name':name, 'description':description}
    _redis.lpush(constants.Keys.MOD_LIST,json.dumps(modInfo))


def GetModules():
    _redis = Redis(host=constants.MOD_REG_SERVER)
    modList = _redis.lrange(constants.Keys.MOD_LIST,0,-1)
    modList = json.loads(modList)
    return modList


from redis import Redis
import json

class SharedVars(Redis):
    """This class abstracts the reading and setting of shared variables between modules.
    This specific implementation uses Redis"""
    def __init__(self,moduleName,*args,**kwargs):
        self._client = Redis(*args,**kwargs)
        self._modName = moduleName

    def getKey(self,varName):
        """generate the key name used to store the var in Redis"""
        return '{}/{}'.format(self._modName,varName)

    def get(self,varName,defaultValue=None):
        """ Get the value of a shared variable.
        @param: varName The name of the variable that you need to access.
        @param: defaultValue The default value to return if the variable doesn't exist yet.
        """
        v = self._client.get(self.getKey(varName))
        if v is None: return defaultValue
        v = v.decode('utf-8')
        v = json.loads(v)
        return v

    def set(self,varName,value):
        """ Set the value of a shared variable.

        @param: varName The name of the variable to set.
        @param: value The new value of the variable.
        """
        self._client.set(self.getKey(varName),json.dumps(value))

    
